package net.minecraft.block;

import com.google.common.base.Predicate;
import java.util.List;
import java.util.Random;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItemFrame;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityComparator;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.IStringSerializable;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockRedstoneComparator extends BlockRedstoneDiode implements ITileEntityProvider
{
    public static final PropertyBool field_176464_a = PropertyBool.create("powered");
    public static final PropertyEnum field_176463_b = PropertyEnum.create("mode", BlockRedstoneComparator.Mode.class);
    private static final String __OBFID = "CL_00000220";

    public BlockRedstoneComparator(boolean p_i45399_1_)
    {
        super(p_i45399_1_);
        this.setDefaultState(this.blockState.getBaseState().withProperty(FACING, EnumFacing.NORTH).withProperty(field_176464_a, Boolean.valueOf(false)).withProperty(field_176463_b, BlockRedstoneComparator.Mode.COMPARE));
        this.isBlockContainer = true;
    }

    /**
     * Get the Item that this Block should drop when harvested.
     *  
     * @param fortune the level of the Fortune enchantment on the player's tool
     */
    public Item getItemDropped(IBlockState state, Random rand, int fortune)
    {
        return Items.comparator;
    }

    @SideOnly(Side.CLIENT)
    public Item getItem(World worldIn, BlockPos pos)
    {
        return Items.comparator;
    }

    protected int func_176403_d(IBlockState p_176403_1_)
    {
        return 2;
    }

    protected IBlockState func_180674_e(IBlockState p_180674_1_)
    {
        Boolean obool = (Boolean)p_180674_1_.getValue(field_176464_a);
        BlockRedstoneComparator.Mode mode = (BlockRedstoneComparator.Mode)p_180674_1_.getValue(field_176463_b);
        EnumFacing enumfacing = (EnumFacing)p_180674_1_.getValue(FACING);
        return Blocks.powered_comparator.getDefaultState().withProperty(FACING, enumfacing).withProperty(field_176464_a, obool).withProperty(field_176463_b, mode);
    }

    protected IBlockState func_180675_k(IBlockState p_180675_1_)
    {
        Boolean obool = (Boolean)p_180675_1_.getValue(field_176464_a);
        BlockRedstoneComparator.Mode mode = (BlockRedstoneComparator.Mode)p_180675_1_.getValue(field_176463_b);
        EnumFacing enumfacing = (EnumFacing)p_180675_1_.getValue(FACING);
        return Blocks.unpowered_comparator.getDefaultState().withProperty(FACING, enumfacing).withProperty(field_176464_a, obool).withProperty(field_176463_b, mode);
    }

    protected boolean func_176406_l(IBlockState p_176406_1_)
    {
        return this.isRepeaterPowered || ((Boolean)p_176406_1_.getValue(field_176464_a)).booleanValue();
    }

    protected int func_176408_a(IBlockAccess p_176408_1_, BlockPos p_176408_2_, IBlockState p_176408_3_)
    {
        TileEntity tileentity = p_176408_1_.getTileEntity(p_176408_2_);
        return tileentity instanceof TileEntityComparator ? ((TileEntityComparator)tileentity).getOutputSignal() : 0;
    }

    private int func_176460_j(World worldIn, BlockPos p_176460_2_, IBlockState p_176460_3_)
    {
        return p_176460_3_.getValue(field_176463_b) == BlockRedstoneComparator.Mode.SUBTRACT ? Math.max(this.func_176397_f(worldIn, p_176460_2_, p_176460_3_) - this.func_176407_c(worldIn, p_176460_2_, p_176460_3_), 0) : this.func_176397_f(worldIn, p_176460_2_, p_176460_3_);
    }

    protected boolean func_176404_e(World worldIn, BlockPos p_176404_2_, IBlockState p_176404_3_)
    {
        int i = this.func_176397_f(worldIn, p_176404_2_, p_176404_3_);

        if (i >= 15)
        {
            return true;
        }
        else if (i == 0)
        {
            return false;
        }
        else
        {
            int j = this.func_176407_c(worldIn, p_176404_2_, p_176404_3_);
            return j == 0 ? true : i >= j;
        }
    }

    protected int func_176397_f(World worldIn, BlockPos p_176397_2_, IBlockState p_176397_3_)
    {
        int i = super.func_176397_f(worldIn, p_176397_2_, p_176397_3_);
        EnumFacing enumfacing = (EnumFacing)p_176397_3_.getValue(FACING);
        BlockPos blockpos1 = p_176397_2_.offset(enumfacing);
        Block block = worldIn.getBlockState(blockpos1).getBlock();

        if (block.hasComparatorInputOverride())
        {
            i = block.getComparatorInputOverride(worldIn, blockpos1);
        }
        else if (i < 15 && block.isNormalCube())
        {
            blockpos1 = blockpos1.offset(enumfacing);
            block = worldIn.getBlockState(blockpos1).getBlock();

            if (block.hasComparatorInputOverride())
            {
                i = block.getComparatorInputOverride(worldIn, blockpos1);
            }
            else if (block.getMaterial() == Material.air)
            {
                EntityItemFrame entityitemframe = this.func_176461_a(worldIn, enumfacing, blockpos1);

                if (entityitemframe != null)
                {
                    i = entityitemframe.func_174866_q();
                }
            }
        }

        return i;
    }

    private EntityItemFrame func_176461_a(World worldIn, final EnumFacing p_176461_2_, BlockPos p_176461_3_)
    {
        List list = worldIn.func_175647_a(EntityItemFrame.class, new AxisAlignedBB((double)p_176461_3_.getX(), (double)p_176461_3_.getY(), (double)p_176461_3_.getZ(), (double)(p_176461_3_.getX() + 1), (double)(p_176461_3_.getY() + 1), (double)(p_176461_3_.getZ() + 1)), new Predicate()
        {
            private static final String __OBFID = "CL_00002129";
            public boolean func_180416_a(Entity p_180416_1_)
            {
                return p_180416_1_ != null && p_180416_1_.func_174811_aO() == p_176461_2_;
            }
            public boolean apply(Object p_apply_1_)
            {
                return this.func_180416_a((Entity)p_apply_1_);
            }
        });
        return list.size() == 1 ? (EntityItemFrame)list.get(0) : null;
    }

    public boolean onBlockActivated(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumFacing side, float hitX, float hitY, float hitZ)
    {
        if (!playerIn.capabilities.allowEdit)
        {
            return false;
        }
        else
        {
            state = state.cycleProperty(field_176463_b);
            worldIn.playSoundEffect((double)pos.getX() + 0.5D, (double)pos.getY() + 0.5D, (double)pos.getZ() + 0.5D, "random.click", 0.3F, state.getValue(field_176463_b) == BlockRedstoneComparator.Mode.SUBTRACT ? 0.55F : 0.5F);
            worldIn.setBlockState(pos, state, 2);
            this.func_176462_k(worldIn, pos, state);
            return true;
        }
    }

    protected void func_176398_g(World worldIn, BlockPos p_176398_2_, IBlockState p_176398_3_)
    {
        if (!worldIn.isBlockTickPending(p_176398_2_, this))
        {
            int i = this.func_176460_j(worldIn, p_176398_2_, p_176398_3_);
            TileEntity tileentity = worldIn.getTileEntity(p_176398_2_);
            int j = tileentity instanceof TileEntityComparator ? ((TileEntityComparator)tileentity).getOutputSignal() : 0;

            if (i != j || this.func_176406_l(p_176398_3_) != this.func_176404_e(worldIn, p_176398_2_, p_176398_3_))
            {
                if (this.func_176402_i(worldIn, p_176398_2_, p_176398_3_))
                {
                    worldIn.func_175654_a(p_176398_2_, this, 2, -1);
                }
                else
                {
                    worldIn.func_175654_a(p_176398_2_, this, 2, 0);
                }
            }
        }
    }

    private void func_176462_k(World worldIn, BlockPos p_176462_2_, IBlockState p_176462_3_)
    {
        int i = this.func_176460_j(worldIn, p_176462_2_, p_176462_3_);
        TileEntity tileentity = worldIn.getTileEntity(p_176462_2_);
        int j = 0;

        if (tileentity instanceof TileEntityComparator)
        {
            TileEntityComparator tileentitycomparator = (TileEntityComparator)tileentity;
            j = tileentitycomparator.getOutputSignal();
            tileentitycomparator.setOutputSignal(i);
        }

        if (j != i || p_176462_3_.getValue(field_176463_b) == BlockRedstoneComparator.Mode.COMPARE)
        {
            boolean flag1 = this.func_176404_e(worldIn, p_176462_2_, p_176462_3_);
            boolean flag = this.func_176406_l(p_176462_3_);

            if (flag && !flag1)
            {
                worldIn.setBlockState(p_176462_2_, p_176462_3_.withProperty(field_176464_a, Boolean.valueOf(false)), 2);
            }
            else if (!flag && flag1)
            {
                worldIn.setBlockState(p_176462_2_, p_176462_3_.withProperty(field_176464_a, Boolean.valueOf(true)), 2);
            }

            this.func_176400_h(worldIn, p_176462_2_, p_176462_3_);
        }
    }

    public void updateTick(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (this.isRepeaterPowered)
        {
            worldIn.setBlockState(pos, this.func_180675_k(state).withProperty(field_176464_a, Boolean.valueOf(true)), 4);
        }

        this.func_176462_k(worldIn, pos, state);
    }

    public void onBlockAdded(World worldIn, BlockPos pos, IBlockState state)
    {
        super.onBlockAdded(worldIn, pos, state);
        worldIn.setTileEntity(pos, this.createNewTileEntity(worldIn, 0));
    }

    public void breakBlock(World worldIn, BlockPos pos, IBlockState state)
    {
        super.breakBlock(worldIn, pos, state);
        worldIn.removeTileEntity(pos);
        this.func_176400_h(worldIn, pos, state);
    }

    /**
     * Called on both Client and Server when World#addBlockEvent is called
     */
    public boolean onBlockEventReceived(World worldIn, BlockPos pos, IBlockState state, int eventID, int eventParam)
    {
        super.onBlockEventReceived(worldIn, pos, state, eventID, eventParam);
        TileEntity tileentity = worldIn.getTileEntity(pos);
        return tileentity == null ? false : tileentity.receiveClientEvent(eventID, eventParam);
    }

    /**
     * Returns a new instance of a block's tile entity class. Called on placing the block.
     */
    public TileEntity createNewTileEntity(World worldIn, int meta)
    {
        return new TileEntityComparator();
    }

    /**
     * Convert the given metadata into a BlockState for this Block
     */
    public IBlockState getStateFromMeta(int meta)
    {
        return this.getDefaultState().withProperty(FACING, EnumFacing.getHorizontal(meta)).withProperty(field_176464_a, Boolean.valueOf((meta & 8) > 0)).withProperty(field_176463_b, (meta & 4) > 0 ? BlockRedstoneComparator.Mode.SUBTRACT : BlockRedstoneComparator.Mode.COMPARE);
    }

    /**
     * Convert the BlockState into the correct metadata value
     */
    public int getMetaFromState(IBlockState state)
    {
        byte b0 = 0;
        int i = b0 | ((EnumFacing)state.getValue(FACING)).getHorizontalIndex();

        if (((Boolean)state.getValue(field_176464_a)).booleanValue())
        {
            i |= 8;
        }

        if (state.getValue(field_176463_b) == BlockRedstoneComparator.Mode.SUBTRACT)
        {
            i |= 4;
        }

        return i;
    }

    protected BlockState createBlockState()
    {
        return new BlockState(this, new IProperty[] {FACING, field_176463_b, field_176464_a});
    }

    public IBlockState onBlockPlaced(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        return this.getDefaultState().withProperty(FACING, placer.func_174811_aO().getOpposite()).withProperty(field_176464_a, Boolean.valueOf(false)).withProperty(field_176463_b, BlockRedstoneComparator.Mode.COMPARE);
    }

    @Override
    public void onNeighborChange(IBlockAccess world, BlockPos pos, BlockPos neighbor)
    {
        if (pos.getY() == neighbor.getY() && world instanceof World)
        {
            onNeighborBlockChange((World)world, pos, world.getBlockState(neighbor), world.getBlockState(neighbor).getBlock());
        }
    }

    @Override
    public boolean getWeakChanges(IBlockAccess world, BlockPos pos)
    {
        return true;
    }

    public static enum Mode implements IStringSerializable
    {
        COMPARE("compare"),
        SUBTRACT("subtract");
        private final String field_177041_c;

        private static final String __OBFID = "CL_00002128";

        private Mode(String p_i45731_3_)
        {
            this.field_177041_c = p_i45731_3_;
        }

        public String toString()
        {
            return this.field_177041_c;
        }

        public String getName()
        {
            return this.field_177041_c;
        }
    }
}