package net.minecraft.block;

import java.util.Random;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumWorldBlockLayer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public abstract class BlockRedstoneDiode extends BlockDirectional
{
    /** Tells whether the repeater is powered or not */
    protected final boolean isRepeaterPowered;
    private static final String __OBFID = "CL_00000226";

    protected BlockRedstoneDiode(boolean p_i45400_1_)
    {
        super(Material.circuits);
        this.isRepeaterPowered = p_i45400_1_;
        this.setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.125F, 1.0F);
    }

    public boolean isFullCube()
    {
        return false;
    }

    public boolean canPlaceBlockAt(World worldIn, BlockPos pos)
    {
        return World.doesBlockHaveSolidTopSurface(worldIn, pos.offsetDown()) ? super.canPlaceBlockAt(worldIn, pos) : false;
    }

    public boolean func_176409_d(World worldIn, BlockPos p_176409_2_)
    {
        return World.doesBlockHaveSolidTopSurface(worldIn, p_176409_2_.offsetDown());
    }

    /**
     * Called randomly when setTickRandomly is set to true (used by e.g. crops to grow, etc.)
     */
    public void randomTick(World worldIn, BlockPos pos, IBlockState state, Random random) {}

    public void updateTick(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!this.func_176405_b(worldIn, pos, state))
        {
            boolean flag = this.func_176404_e(worldIn, pos, state);

            if (this.isRepeaterPowered && !flag)
            {
                worldIn.setBlockState(pos, this.func_180675_k(state), 2);
            }
            else if (!this.isRepeaterPowered)
            {
                worldIn.setBlockState(pos, this.func_180674_e(state), 2);

                if (!flag)
                {
                    worldIn.func_175654_a(pos, this.func_180674_e(state).getBlock(), this.func_176399_m(state), -1);
                }
            }
        }
    }

    @SideOnly(Side.CLIENT)
    public boolean shouldSideBeRendered(IBlockAccess worldIn, BlockPos pos, EnumFacing side)
    {
        return side.getAxis() != EnumFacing.Axis.Y;
    }

    protected boolean func_176406_l(IBlockState p_176406_1_)
    {
        return this.isRepeaterPowered;
    }

    public int isProvidingStrongPower(IBlockAccess worldIn, BlockPos pos, IBlockState state, EnumFacing side)
    {
        return this.isProvidingWeakPower(worldIn, pos, state, side);
    }

    public int isProvidingWeakPower(IBlockAccess worldIn, BlockPos pos, IBlockState state, EnumFacing side)
    {
        return !this.func_176406_l(state) ? 0 : (state.getValue(FACING) == side ? this.func_176408_a(worldIn, pos, state) : 0);
    }

    public void onNeighborBlockChange(World worldIn, BlockPos pos, IBlockState state, Block neighborBlock)
    {
        if (this.func_176409_d(worldIn, pos))
        {
            this.func_176398_g(worldIn, pos, state);
        }
        else
        {
            this.dropBlockAsItem(worldIn, pos, state, 0);
            worldIn.setBlockToAir(pos);
            EnumFacing[] aenumfacing = EnumFacing.values();
            int i = aenumfacing.length;

            for (int j = 0; j < i; ++j)
            {
                EnumFacing enumfacing = aenumfacing[j];
                worldIn.notifyNeighborsOfStateChange(pos.offset(enumfacing), this);
            }
        }
    }

    protected void func_176398_g(World worldIn, BlockPos p_176398_2_, IBlockState p_176398_3_)
    {
        if (!this.func_176405_b(worldIn, p_176398_2_, p_176398_3_))
        {
            boolean flag = this.func_176404_e(worldIn, p_176398_2_, p_176398_3_);

            if ((this.isRepeaterPowered && !flag || !this.isRepeaterPowered && flag) && !worldIn.isBlockTickPending(p_176398_2_, this))
            {
                byte b0 = -1;

                if (this.func_176402_i(worldIn, p_176398_2_, p_176398_3_))
                {
                    b0 = -3;
                }
                else if (this.isRepeaterPowered)
                {
                    b0 = -2;
                }

                worldIn.func_175654_a(p_176398_2_, this, this.func_176403_d(p_176398_3_), b0);
            }
        }
    }

    public boolean func_176405_b(IBlockAccess p_176405_1_, BlockPos p_176405_2_, IBlockState p_176405_3_)
    {
        return false;
    }

    protected boolean func_176404_e(World worldIn, BlockPos p_176404_2_, IBlockState p_176404_3_)
    {
        return this.func_176397_f(worldIn, p_176404_2_, p_176404_3_) > 0;
    }

    protected int func_176397_f(World worldIn, BlockPos p_176397_2_, IBlockState p_176397_3_)
    {
        EnumFacing enumfacing = (EnumFacing)p_176397_3_.getValue(FACING);
        BlockPos blockpos1 = p_176397_2_.offset(enumfacing);
        int i = worldIn.getRedstonePower(blockpos1, enumfacing);

        if (i >= 15)
        {
            return i;
        }
        else
        {
            IBlockState iblockstate1 = worldIn.getBlockState(blockpos1);
            return Math.max(i, iblockstate1.getBlock() == Blocks.redstone_wire ? ((Integer)iblockstate1.getValue(BlockRedstoneWire.POWER)).intValue() : 0);
        }
    }

    protected int func_176407_c(IBlockAccess p_176407_1_, BlockPos p_176407_2_, IBlockState p_176407_3_)
    {
        EnumFacing enumfacing = (EnumFacing)p_176407_3_.getValue(FACING);
        EnumFacing enumfacing1 = enumfacing.rotateY();
        EnumFacing enumfacing2 = enumfacing.rotateYCCW();
        return Math.max(this.func_176401_c(p_176407_1_, p_176407_2_.offset(enumfacing1), enumfacing1), this.func_176401_c(p_176407_1_, p_176407_2_.offset(enumfacing2), enumfacing2));
    }

    protected int func_176401_c(IBlockAccess p_176401_1_, BlockPos p_176401_2_, EnumFacing p_176401_3_)
    {
        IBlockState iblockstate = p_176401_1_.getBlockState(p_176401_2_);
        Block block = iblockstate.getBlock();
        return this.func_149908_a(block) ? (block == Blocks.redstone_wire ? ((Integer)iblockstate.getValue(BlockRedstoneWire.POWER)).intValue() : p_176401_1_.getStrongPower(p_176401_2_, p_176401_3_)) : 0;
    }

    /**
     * Can this block provide power. Only wire currently seems to have this change based on its state.
     */
    public boolean canProvidePower()
    {
        return true;
    }

    public IBlockState onBlockPlaced(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        return this.getDefaultState().withProperty(FACING, placer.func_174811_aO().getOpposite());
    }

    public void onBlockPlacedBy(World worldIn, BlockPos pos, IBlockState state, EntityLivingBase placer, ItemStack stack)
    {
        if (this.func_176404_e(worldIn, pos, state))
        {
            worldIn.scheduleUpdate(pos, this, 1);
        }
    }

    public void onBlockAdded(World worldIn, BlockPos pos, IBlockState state)
    {
        this.func_176400_h(worldIn, pos, state);
    }

    protected void func_176400_h(World worldIn, BlockPos p_176400_2_, IBlockState p_176400_3_)
    {
        EnumFacing enumfacing = (EnumFacing)p_176400_3_.getValue(FACING);
        BlockPos blockpos1 = p_176400_2_.offset(enumfacing.getOpposite());
        worldIn.notifyBlockOfStateChange(blockpos1, this);
        worldIn.notifyNeighborsOfStateExcept(blockpos1, this, enumfacing);
    }

    /**
     * Called when a player destroys this Block
     */
    public void onBlockDestroyedByPlayer(World worldIn, BlockPos pos, IBlockState state)
    {
        if (this.isRepeaterPowered)
        {
            EnumFacing[] aenumfacing = EnumFacing.values();
            int i = aenumfacing.length;

            for (int j = 0; j < i; ++j)
            {
                EnumFacing enumfacing = aenumfacing[j];
                worldIn.notifyNeighborsOfStateChange(pos.offset(enumfacing), this);
            }
        }

        super.onBlockDestroyedByPlayer(worldIn, pos, state);
    }

    public boolean isOpaqueCube()
    {
        return false;
    }

    protected boolean func_149908_a(Block p_149908_1_)
    {
        return p_149908_1_.canProvidePower();
    }

    protected int func_176408_a(IBlockAccess p_176408_1_, BlockPos p_176408_2_, IBlockState p_176408_3_)
    {
        return 15;
    }

    public static boolean isRedstoneRepeaterBlockID(Block p_149909_0_)
    {
        return Blocks.unpowered_repeater.func_149907_e(p_149909_0_) || Blocks.unpowered_comparator.func_149907_e(p_149909_0_);
    }

    public boolean func_149907_e(Block p_149907_1_)
    {
        return p_149907_1_ == this.func_180674_e(this.getDefaultState()).getBlock() || p_149907_1_ == this.func_180675_k(this.getDefaultState()).getBlock();
    }

    public boolean func_176402_i(World worldIn, BlockPos p_176402_2_, IBlockState p_176402_3_)
    {
        EnumFacing enumfacing = ((EnumFacing)p_176402_3_.getValue(FACING)).getOpposite();
        BlockPos blockpos1 = p_176402_2_.offset(enumfacing);
        return isRedstoneRepeaterBlockID(worldIn.getBlockState(blockpos1).getBlock()) ? worldIn.getBlockState(blockpos1).getValue(FACING) != enumfacing : false;
    }

    protected int func_176399_m(IBlockState p_176399_1_)
    {
        return this.func_176403_d(p_176399_1_);
    }

    protected abstract int func_176403_d(IBlockState p_176403_1_);

    protected abstract IBlockState func_180674_e(IBlockState p_180674_1_);

    protected abstract IBlockState func_180675_k(IBlockState p_180675_1_);

    public boolean isAssociatedBlock(Block other)
    {
        return this.func_149907_e(other);
    }

    @SideOnly(Side.CLIENT)
    public EnumWorldBlockLayer getBlockLayer()
    {
        return EnumWorldBlockLayer.CUTOUT;
    }
}