package net.minecraft.util;

import com.google.common.collect.AbstractIterator;
import java.util.Iterator;
import net.minecraft.entity.Entity;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockPos extends Vec3i
{
    /** The BlockPos with all coordinates 0 */
    public static final BlockPos ORIGIN = new BlockPos(0, 0, 0);
    private static final int field_177990_b = 1 + MathHelper.calculateLogBaseTwo(MathHelper.roundUpToPowerOfTwo(30000000));
    private static final int field_177991_c = field_177990_b;
    private static final int field_177989_d = 64 - field_177990_b - field_177991_c;
    private static final int field_177987_f = 0 + field_177991_c;
    private static final int field_177988_g = field_177987_f + field_177989_d;
    private static final long field_177994_h = (1L << field_177990_b) - 1L;
    private static final long field_177995_i = (1L << field_177989_d) - 1L;
    private static final long field_177993_j = (1L << field_177991_c) - 1L;
    private static final String __OBFID = "CL_00002334";

    public BlockPos(int x, int y, int z)
    {
        super(x, y, z);
    }

    public BlockPos(double x, double y, double z)
    {
        super(x, y, z);
    }

    public BlockPos(Entity p_i46032_1_)
    {
        this(p_i46032_1_.posX, p_i46032_1_.posY, p_i46032_1_.posZ);
    }

    public BlockPos(Vec3 p_i46033_1_)
    {
        this(p_i46033_1_.xCoord, p_i46033_1_.yCoord, p_i46033_1_.zCoord);
    }

    public BlockPos(Vec3i p_i46034_1_)
    {
        this(p_i46034_1_.getX(), p_i46034_1_.getY(), p_i46034_1_.getZ());
    }

    /**
     * Add the given coordinates to the coordinates of this BlockPos
     *  
     * @param x X coordinate
     * @param y Y coordinate
     * @param z Z coordinate
     */
    public BlockPos add(double x, double y, double z)
    {
        return new BlockPos((double)this.getX() + x, (double)this.getY() + y, (double)this.getZ() + z);
    }

    /**
     * Add the given coordinates to the coordinates of this BlockPos
     *  
     * @param x X coordinate
     * @param y Y coordinate
     * @param z Z coordinate
     */
    public BlockPos add(int x, int y, int z)
    {
        return new BlockPos(this.getX() + x, this.getY() + y, this.getZ() + z);
    }

    /**
     * Add the given Vector to this BlockPos
     */
    public BlockPos add(Vec3i vec)
    {
        return new BlockPos(this.getX() + vec.getX(), this.getY() + vec.getY(), this.getZ() + vec.getZ());
    }

    /**
     * Multiply every coordinate by the given factor
     */
    public BlockPos multiply(int factor)
    {
        return new BlockPos(this.getX() * factor, this.getY() * factor, this.getZ() * factor);
    }

    /**
     * Offset this BlockPos 1 block up
     */
    public BlockPos offsetUp()
    {
        return this.offsetUp(1);
    }

    /**
     * Subtract the given Vector from this BlockPos
     */
    @SideOnly(Side.CLIENT)
    public BlockPos subtract(Vec3i vec)
    {
        return new BlockPos(this.getX() - vec.getX(), this.getY() - vec.getY(), this.getZ() - vec.getZ());
    }

    /**
     * Offset this BlockPos n blocks up
     */
    public BlockPos offsetUp(int n)
    {
        return this.offset(EnumFacing.UP, n);
    }

    /**
     * Offset this BlockPos 1 block down
     */
    public BlockPos offsetDown()
    {
        return this.offsetDown(1);
    }

    /**
     * Offset this BlockPos n blocks down
     */
    public BlockPos offsetDown(int n)
    {
        return this.offset(EnumFacing.DOWN, n);
    }

    /**
     * Offset this BlockPos 1 block in northern direction
     */
    public BlockPos offsetNorth()
    {
        return this.offsetNorth(1);
    }

    /**
     * Offset this BlockPos n blocks in northern direction
     */
    public BlockPos offsetNorth(int n)
    {
        return this.offset(EnumFacing.NORTH, n);
    }

    /**
     * Offset this BlockPos 1 block in southern direction
     */
    public BlockPos offsetSouth()
    {
        return this.offsetSouth(1);
    }

    /**
     * Offset this BlockPos n blocks in southern direction
     */
    public BlockPos offsetSouth(int n)
    {
        return this.offset(EnumFacing.SOUTH, n);
    }

    /**
     * Offset this BlockPos 1 block in western direction
     */
    public BlockPos offsetWest()
    {
        return this.offsetWest(1);
    }

    /**
     * Offset this BlockPos n blocks in western direction
     */
    public BlockPos offsetWest(int n)
    {
        return this.offset(EnumFacing.WEST, n);
    }

    /**
     * Offset this BlockPos 1 block in eastern direction
     */
    public BlockPos offsetEast()
    {
        return this.offsetEast(1);
    }

    /**
     * Offset this BlockPos n blocks in eastern direction
     */
    public BlockPos offsetEast(int n)
    {
        return this.offset(EnumFacing.EAST, n);
    }

    /**
     * Offset this BlockPos 1 block in the given direction
     */
    public BlockPos offset(EnumFacing facing)
    {
        return this.offset(facing, 1);
    }

    /**
     * Offset this BlockPos n blocks in the given direction
     */
    public BlockPos offset(EnumFacing facing, int n)
    {
        return new BlockPos(this.getX() + facing.getFrontOffsetX() * n, this.getY() + facing.getFrontOffsetY() * n, this.getZ() + facing.getFrontOffsetZ() * n);
    }

    /**
     * Calculate the cross product of this BlockPos and the given Vector. Version of crossProduct that returns a
     * BlockPos instead of a Vec3i
     */
    public BlockPos crossProductBP(Vec3i vec)
    {
        return new BlockPos(this.getY() * vec.getZ() - this.getZ() * vec.getY(), this.getZ() * vec.getX() - this.getX() * vec.getZ(), this.getX() * vec.getY() - this.getY() * vec.getX());
    }

    /**
     * Serialize this BlockPos into a long value
     */
    public long toLong()
    {
        return ((long)this.getX() & field_177994_h) << field_177988_g | ((long)this.getY() & field_177995_i) << field_177987_f | ((long)this.getZ() & field_177993_j) << 0;
    }

    /**
     * Create a BlockPos from a serialized long value (created by toLong)
     */
    public static BlockPos fromLong(long serialized)
    {
        int j = (int)(serialized << 64 - field_177988_g - field_177990_b >> 64 - field_177990_b);
        int k = (int)(serialized << 64 - field_177987_f - field_177989_d >> 64 - field_177989_d);
        int l = (int)(serialized << 64 - field_177991_c >> 64 - field_177991_c);
        return new BlockPos(j, k, l);
    }

    /**
     * Create an Iterable that returns all positions in the box specified by the given corners
     *  
     * @param from The first corner (inclusive)
     * @param to the second corner (exclusive)
     */
    public static Iterable getAllInBox(BlockPos from, BlockPos to)
    {
        final BlockPos blockpos2 = new BlockPos(Math.min(from.getX(), to.getX()), Math.min(from.getY(), to.getY()), Math.min(from.getZ(), to.getZ()));
        final BlockPos blockpos3 = new BlockPos(Math.max(from.getX(), to.getX()), Math.max(from.getY(), to.getY()), Math.max(from.getZ(), to.getZ()));
        return new Iterable()
        {
            private static final String __OBFID = "CL_00002333";
            public Iterator iterator()
            {
                return new AbstractIterator()
                {
                    private BlockPos lastReturned = null;
                    private static final String __OBFID = "CL_00002332";
                    /**
                     * Synthetic method called by computeNext
                     */
                    protected BlockPos computeNext0()
                    {
                        if (this.lastReturned == null)
                        {
                            this.lastReturned = blockpos2;
                            return this.lastReturned;
                        }
                        else if (this.lastReturned.equals(blockpos3))
                        {
                            return (BlockPos)this.endOfData();
                        }
                        else
                        {
                            int i = this.lastReturned.getX();
                            int j = this.lastReturned.getY();
                            int k = this.lastReturned.getZ();

                            if (i < blockpos3.getX())
                            {
                                ++i;
                            }
                            else if (j < blockpos3.getY())
                            {
                                i = blockpos2.getX();
                                ++j;
                            }
                            else if (k < blockpos3.getZ())
                            {
                                i = blockpos2.getX();
                                j = blockpos2.getY();
                                ++k;
                            }

                            this.lastReturned = new BlockPos(i, j, k);
                            return this.lastReturned;
                        }
                    }
                    protected Object computeNext()
                    {
                        return this.computeNext0();
                    }
                };
            }
        };
    }

    /**
     * Like getAllInBox but reuses a single MutableBlockPos instead. If this method is used, the resulting BlockPos
     * instances can only be used inside the iteration loop.
     *  
     * @param from The first corner (inclusive)
     * @param to the second corner (exclusive)
     */
    public static Iterable getAllInBoxMutable(BlockPos from, BlockPos to)
    {
        final BlockPos blockpos2 = new BlockPos(Math.min(from.getX(), to.getX()), Math.min(from.getY(), to.getY()), Math.min(from.getZ(), to.getZ()));
        final BlockPos blockpos3 = new BlockPos(Math.max(from.getX(), to.getX()), Math.max(from.getY(), to.getY()), Math.max(from.getZ(), to.getZ()));
        return new Iterable()
        {
            private static final String __OBFID = "CL_00002331";
            public Iterator iterator()
            {
                return new AbstractIterator()
                {
                    private BlockPos.MutableBlockPos theBlockPos = null;
                    private static final String __OBFID = "CL_00002330";
                    /**
                     * Synthetic method called by computeNext
                     */
                    protected BlockPos.MutableBlockPos computeNext0()
                    {
                        if (this.theBlockPos == null)
                        {
                            this.theBlockPos = new BlockPos.MutableBlockPos(blockpos2.getX(), blockpos2.getY(), blockpos2.getZ(), null);
                            return this.theBlockPos;
                        }
                        else if (this.theBlockPos.equals(blockpos3))
                        {
                            return (BlockPos.MutableBlockPos)this.endOfData();
                        }
                        else
                        {
                            int i = this.theBlockPos.getX();
                            int k = this.theBlockPos.getY();
                            int j = this.theBlockPos.getZ();

                            if (i < blockpos3.getX())
                            {
                                ++i;
                            }
                            else if (k < blockpos3.getY())
                            {
                                i = blockpos2.getX();
                                ++k;
                            }
                            else if (j < blockpos3.getZ())
                            {
                                i = blockpos2.getX();
                                k = blockpos2.getY();
                                ++j;
                            }

                            this.theBlockPos.x = i;
                            this.theBlockPos.y = k;
                            this.theBlockPos.z = j;
                            return this.theBlockPos;
                        }
                    }
                    protected Object computeNext()
                    {
                        return this.computeNext0();
                    }
                };
            }
        };
    }

    /**
     * Calculate the cross product of this and the given Vector
     */
    public Vec3i crossProduct(Vec3i vec)
    {
        return this.crossProductBP(vec);
    }

    public static final class MutableBlockPos extends BlockPos
        {
            /** Mutable X Coordinate */
            public int x;
            /** Mutable Y Coordinate */
            public int y;
            /** Mutable Z Coordinate */
            public int z;
            private static final String __OBFID = "CL_00002329";

            private MutableBlockPos(int x_, int y_, int z_)
            {
                super(0, 0, 0);
                this.x = x_;
                this.y = y_;
                this.z = z_;
            }

            /**
             * Get the X coordinate
             */
            public int getX()
            {
                return this.x;
            }

            /**
             * Get the Y coordinate
             */
            public int getY()
            {
                return this.y;
            }

            /**
             * Get the Z coordinate
             */
            public int getZ()
            {
                return this.z;
            }

            /**
             * Calculate the cross product of this and the given Vector
             */
            public Vec3i crossProduct(Vec3i vec)
            {
                return super.crossProductBP(vec);
            }

            MutableBlockPos(int p_i46025_1_, int p_i46025_2_, int p_i46025_3_, Object p_i46025_4_)
            {
                this(p_i46025_1_, p_i46025_2_, p_i46025_3_);
            }
        }
}